"""
This is an example on how to use the nem module to read data from AEMO.

.. code-block:: python

    import nem

    for document in nem.current.DispatchIS:
    for data in document.filter("PRICE"):
        if data["REGIONID"] == "VIC1":
            print(data["RRP"])
            
    # prints out the RRP for VIC

    print(nem.generators()["GSTONE1"]["Fuel Source - Primary"])
    # 'Fossil'
    # prints out the primary fuel source for gstone
"""

from nem.importer import importer
from nem.importer import document
from nem.stationdata import stationdata

current = importer()
"""Provides a ``nem.importer`` for current data"""
archive = importer(url="http://www.nemweb.com.au/Reports/Archive/")
"""Provides a ``nem.importer`` for archive data"""
historical = importer(url="http://www.nemweb.com.au/Data_Archive/Wholesale_Electricity/MMSDM/", historical=True)
"""Provides a ``nem.importer`` for historical data (10+ years)"""
direct = document
"""Provides a ``nem.importer.document`` for single use file extraction"""


generators = stationdata
"""Dictonary of generators and their data from the partipants list"""


# The request library is used to fetch content through HTTP
import requests
from pyexcel_xls import get_data
from io import BytesIO

class stationdata(dict):
    def __init__(self, url="https://www.aemo.com.au/-/media/Files/Electricity/NEM/Participant_Information/Current-Participants/NEM-Registration-and-Exemption-List.xls"):
        """
        Provides generator information from the AEMO current list of registrations and exemptions list as AEMO does not publish this data as part of the normal dataset.

        :rtype: dictonary
        """
        # parse the xls document for NEM registration details
        stationdata = requests.get(url)
        f = BytesIO()
        f.write(stationdata.content)

        data = get_data(f)
        gen_table = data["Generators and Scheduled Loads"]
        gen_table_headers = gen_table[0]
        for row in gen_table[1:]:
            col_i = 0
            generator = {}
            for col in row:
                generator[gen_table_headers[col_i]] = col
                col_i += 1
            if "DUID" in generator:
                self[generator["DUID"]] = generator


from setuptools import setup
setup(
  name = 'nem',
  packages = ['nem'], # this must be the same as the name above
  version = '0.0.4',
  description = 'Access library for AEMO / NEM goverement data',
  author = 'Michael Wheeler',
  author_email = 'michael@michael-wheeler.org',
  url = 'https://github.com/theskorm/nemweb2', # use the URL to the github repo
  download_url = 'https://github.com/TheSkorm/nemweb2/archive/0.0.4.tar.gz', # I'll explain this in a second
  keywords = ['aemo', 'nem', 'gov', 'au'], # arbitrary keywords
  classifiers = [],
  install_requires = [
        "beautifulsoup4"
        ]
)


import csv
import re
import os
import pytz
from datetime import datetime
from io import BytesIO
from urllib.request import urlopen
from zipfile import ZipFile

from bs4 import BeautifulSoup

OLDEST_YEAR = 2009
OLDEST_MONTH = 7
class importer():
    """Impoter class allows downloading data from AEMO NEM site

    :param url:
        URL to the NEM server
    :type url: ``string``

    :param historical:
        Access to historical data using the MMSDM data
    :type historical: ``boolean``

    :returns: Returns an array of documents
    :rtype: ``[document]``
    """
    def __init__(self, url="http://www.nemweb.com.au/Reports/CURRENT/", historical=False):
        if historical==False:
            self.p5 = files(url + "P5_Reports/", historical)
            """NEM p5 data documents
            
            :rtype: [document]
            """
            self.DispatchIS = files(url + "DispatchIS_Reports/", historical)
            """NEM DispatchIS data documents
            
            :rtype: [document]
            """
            self.Notices = files(url + "Market_Notice/", historical)
            """NEM Notices data documents
            
            :rtype: [document]
            """
            self.SCADA = files(url + "Dispatch_SCADA/", historical)
            """NEM SCADA data documents
            
            :rtype: [document]
            """
            self.CO2 = files(url + "CDEII/", historical)
            """NEM CO2 data documents
            
            :rtype: [document]
            """
        else:
            self.p5 = files(url, "PUBLIC_DVD_P5MIN_REGIONSOLUTION")
            self.DispatchIS = files(url, "PUBLIC_DVD_DISPATCHREGIONSUM")
            # self.Notices = files(url + "Market_Notice/", historical) #TODO
            self.SCADA = files(url, "PUBLIC_DVD_DISPATCH_UNIT_SCADA")
            self.CO2 = files(url, "PUBLIC_DVD_BILLING_CO2E_PUBLICATION") #this is wrong

class files(list):
    """Lists all the documents avaliable for parsing by this module

    :param baseUrl:
        URL to list files for
    :type baseUrl: ``string``

    :param historical:
        Enables generating a list of files for every year, requires being set to the filename requested (eg PUBLIC_DVD_P5MIN_REGIONSOLUTION)
    :type historical: ``string``

    :returns: Returns an array of documents
    :rtype: ``[document]``
    """
    def __init__(self, baseUrl, historical):
        if historical == False:
            indexPage = BeautifulSoup(urlopen(baseUrl).read(), "html.parser")
            for link in indexPage.find_all('a')[1:]:
                url = link.get('href').split("/")[-1]
                if len(url) > 2 and url[-1] != "/":
                    self.append(document(baseUrl + url))
        else:
            for year in range(OLDEST_YEAR,datetime.now().year+1):
                for month in range(1,13):
                    if year == OLDEST_YEAR and month < OLDEST_MONTH: #skip oldest months
                        continue
                    if year == datetime.now().year and  month >= datetime.now().month: # skip months that haven't been stored yet
                        continue


                    url = '{}{}/MMSDM_{}_{:02d}/MMSDM_Historical_Data_SQLLoader/DATA/{}_{}{:02d}010000.zip'.format(baseUrl, year, year, month, historical, year, month)
                    self.append(document(url))
                            


class document():
    """Lists all the documents avaliable for parsing by this module

    :param url:
        URL to parse
    :type url: ``string``
    """
    def __init__(self, url):
        self.url = url
        self._cached = None
        nemDateRegex = re.compile('(2\d{11}(?:\d{2})?)[\._]')
        try:
            strDate = nemDateRegex.findall(url)[0]
            try:
                dateObject = datetime.strptime(strDate, "%Y%m%d%H%M")
            except ValueError:
                try:
                    dateObject = datetime.strptime(strDate, "%Y%m%d")
                except ValueError:
                    dateObject = datetime.strptime(strDate, "%Y%m%d%H%M%S")
            self.dateTime = dateObject.replace(tzinfo=pytz.timezone("Australia/Brisbane"))
            """Provides a datetime object for the file based on the timestamp in the filename

            :rtype: datetime
            """
        except(IndexError):
            pass


    def __str__(self):
        return self.url

    def __repr__(self):
        return self.url

    @property
    def data(self):
        """Provides the raw data from the file

        :rtype: list
        :returns: Array of raw lines of data
        """
        if not self._cached:
            if self.url[-4:].lower() != ".csv":
                self._cached = self._extract(self.url)
            else:
                self._cached = urlopen(self.url).read()
            self._cached = self._cached.decode('utf-8').split("\n")
        return self._cached

    def _extract(self, url):
        file = ZipFile(BytesIO(urlopen(url).read()))
        data = file.read(file.namelist()[0])
        return data
    @property
    def datasets(self):
        """Provides lists of datasets in the file

        :rtype: list
        :returns: Array of datasets avaliable
        """
        csvfile = csv.reader(self.data)
        data = []
        for row in csvfile:
            try:
                if (row[0] == "I"):
                    data.append(row[2])
            except(IndexError):
                pass
        return data
    @property
    def clean_data(self):
        """Provides a dictonary of all the datasets

        :rtype: object
        :returns: {"dataset1":[{"header1":"value"}]}
        """
        csvfile = csv.reader(self.data)
        headers = []
        data = {}
        dataset = ""
        table = ""
        for row in csvfile:
            try:
                if (row[0] == "I" ):
                    headers = row
                    dataset = (row[1] + "-" + row[2]) #prepend with the setp.world variable.
                    data[dataset] = []
                    table = row[2]
                elif row[2] == table:
                    rowCleaned = {headers[ind]: self.string_to_float(x) for ind, x in enumerate(row) if x != ''}
                    data[dataset].append(rowCleaned)

            except(IndexError):
                pass
        return data
    def string_to_float(self,x):
        try:
            x = int(x)
        except ValueError:
            try:
                x = float(x)
            except ValueError:
                pass
            
        return x    
    def filter(self, dataSet):
        """Filters out data to a specific dataset and return that set

        :rtype: [{"header1":"data1"}]
        """
        csvfile = csv.reader(self.data)
        headers = []
        data = []
        for row in csvfile:
            try:
                if (row[0] == "I" and row[2] == dataSet):
                    headers = row
                elif row[2] == dataSet:
                    rowCleaned = {headers[ind]: x for ind, x in enumerate(row) if x != ''}
                    data.append(rowCleaned)
            except(IndexError):
                pass
        return data
